#include "selector.hpp"

Selector::Selector(int x, int y, int type,
		 char *selector_title,
		 int *initial_selection,
		 void (*function)(),
		 char *button1_title,
		 char *button2_title,
		 char *button3_title,
		 char *button4_title,
		 char *button5_title,
		 char *button6_title,
		 char *button7_title,
		 char *button8_title) :

	Icon(x, y, x, y)	// we'll set x2, y2 later

{
	button_legend [0] = button1_title;
	button_legend [1] = button2_title;
	button_legend [2] = button3_title;
	button_legend [3] = button4_title;
	button_legend [4] = button5_title;
	button_legend [5] = button6_title;
	button_legend [6] = button7_title;
	button_legend [7] = button8_title;

	// find out how many buttons the selector has
	number_of_buttons = 0;
	for (int i = 0; i < MAX_SELECTOR_BUTTONS; i++) {
	  if (button_legend [i])
	    number_of_buttons++;
	  else
	    break;
	}

	// keep a copy of the title and activation function
	strncpy(selector_legend,
		selector_title,
		MAX_LEGEND_LENGTH);
	activation_function = function;
	button_selected = initial_selection;

	shade_color = BLACK;
	border_color = WHITE;
	fill_color = GRAY;
	inactive_color = CYAN;
	active_color = BRIGHT_CYAN;
	legend_color = BLACK;

	int width, height;
	if (type == VERTICAL_SELECTOR) {

	  // vertically displayed buttons
	  width = SELECTOR_BUTTON_WIDTH + \
			  SELECTOR_LEFT_MARGIN + \
			  SELECTOR_RIGHT_MARGIN;

	  height = number_of_buttons * SELECTOR_BUTTON_HEIGHT + \
			  SELECTOR_TOP_MARGIN + \
			  SELECTOR_LEGEND_HEIGHT + \
			  SELECTOR_BOTTOM_MARGIN;
	}

	else {

	  // horizontally displayed buttons
	  width = number_of_buttons * SELECTOR_BUTTON_WIDTH + \
			  SELECTOR_LEFT_MARGIN + \
			  SELECTOR_RIGHT_MARGIN;

	  height = SELECTOR_BUTTON_HEIGHT + \
			  SELECTOR_TOP_MARGIN + \
			  SELECTOR_LEGEND_HEIGHT + \
			  SELECTOR_BOTTOM_MARGIN;
	}

	x1 = x;
	y1 = y;
	x2 = x1 + width;
	y2 = y1 + height;

	// dynamically update width of Icon
	Icon::x2 = x2;
	Icon::y2 = y2;

	selector_box = new ShadedBox(x1, y1,
					width, height,
					shade_color,
					border_color,
					fill_color);

	for (i = 0; i < number_of_buttons; i++) {
	  if (type == VERTICAL_SELECTOR) {
	    // show buttons vertically
	    button [i] = new Button(this, i,
		x1 + SELECTOR_LEFT_MARGIN,
		y2 - SELECTOR_TOP_MARGIN - \
		  (i + 1) * SELECTOR_BUTTON_HEIGHT,
		SELECTOR_BUTTON_WIDTH,
		SELECTOR_BUTTON_HEIGHT,
		button_legend [i],
		shade_color,
		inactive_color,
		active_color,
		BLACK);
	  }
	  else {
	    // show buttons horizontally
	    button [i] = new Button(this, i,
			x1 + SELECTOR_LEFT_MARGIN + \
			  i * SELECTOR_BUTTON_WIDTH,
			y1 + SELECTOR_LEGEND_HEIGHT +
			  SELECTOR_TOP_MARGIN,
			SELECTOR_BUTTON_WIDTH,
			SELECTOR_BUTTON_HEIGHT,
			button_legend [i],
			shade_color,
			inactive_color,
			active_color,
			BLACK);
	  }
	}
}


void Selector::Display()
{
	selector_box->Display();
	for (int i = 0; i < number_of_buttons; i++) {
	  if (i == *button_selected)
	    button [i]->Highlight();
	  else
	  button [i]->Display();
	}

	// partition off the legend area
	BOX_TYPE line;
	line.x1 = x1 + 2;
	line.y1 = y1 + SELECTOR_LEGEND_HEIGHT + 2;
	line.x2 = x2 - 2;
	line.y2 = line.y1;
	GraphicsHandler->DrawLine(border_color,
				  SOLID_LINE,
				  &line);

	// display the selector legend
	BOX_TYPE box;
	box.x1 = x1;
	box.y1 = y1 + 2;
	box.x2 = x2;
	box.y2 = y1 + SELECTOR_LEGEND_HEIGHT;
	Legend title_legend;
	title_legend.Center(&box,
			    legend_color,
			    selector_legend);
}

void Selector::Select(int selection)
{
	// abort if invalid selections attempted
	if (selection >= number_of_buttons) return;
	if (selection < 0) return;

	// capture the selection
	*button_selected = selection;

	// turn on the selection, turn off everything
	// else
	for (int i = 0; i < number_of_buttons; i++) {
	  if (i == selection)
	     button [i]->Color(active_color);
	  else
	     button [i]->Color(inactive_color);
	  button [i]->Display();
	}
	// invoke the activation function, so the
	// application program can execute the command
	if (activation_function)
	  (*activation_function)();
}

Selector::~Selector()
{
	delete selector_box;

	for (int i = 0; i < number_of_buttons; i++)
	  delete button [i];
}


Button::Button(Selector* parent, int number,
			     int cx, int cy,
			     int width, int height,
			     char* button_legend,
			     COLOR_TYPE border_clr,
			     COLOR_TYPE inactive_clr,
			     COLOR_TYPE active_clr,
			     COLOR_TYPE legend_clr) :
	Icon(cx + 1, cy + 1,
	     cx + width - 1, cy + height - 1)
{
	x1 = cx + 1;
	y1 = cy + 1;
	x2 = x1 + width - 2;
	y2 = y1 + height - 2;
	parent_selector = parent;
	SetLegend(button_legend);
	button_number = number;
	border_color = border_clr;
	active_color = active_clr;
	inactive_color = inactive_clr;
	fill_color = inactive_color;
	legend_color = legend_clr;
	box = new Box(x1, y1, x2 - x1, y2 - y1,
			border_color,
			fill_color);
}

void Button::Display()
{

	box->Display();

	// display the switch legend
	BOX_TYPE box;
	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;
	Legend title_legend;
	title_legend.Center(&box, legend_color, legend);
}

void Button::Color(COLOR_TYPE color)
{
	fill_color = color;
	box->Color(fill_color);
}

void Button::Highlight()
{
	fill_color = active_color;
	box->Color(fill_color);
	Display();
}

void Button::UnHighlight()
{
	fill_color = inactive_color;
	box->Color(fill_color);
	Display();
}

void Button::LeftButtonPressed()
{
	parent_selector->Select(button_number);
}


Button::~Button()
{
	delete box;
}



HorizontalSelector::HorizontalSelector(int bx, int by,
		 char *selector_title,
		 int *initial_selection,
		 void (*function)(),
		 char *button1_title,
		 char *button2_title,
		 char *button3_title,
		 char *button4_title,
		 char *button5_title,
		 char *button6_title,
		 char *button7_title,
		 char *button8_title) :

	Selector(bx, by, HORIZONTAL_SELECTOR,
	selector_title, initial_selection,
	function,
	button1_title,
	button2_title,
	button3_title,
	button4_title,
	button5_title,
	button6_title,
	button7_title,
	button8_title)
{}



VerticalSelector::VerticalSelector(
		 int bx, int by,
		 char *selector_title,
		 int *initial_selection,
		 void (*function)(),
		 char *button1_title,
		 char *button2_title,
		 char *button3_title,
		 char *button4_title,
		 char *button5_title,
		 char *button6_title,
		 char *button7_title,
		 char *button8_title) :

	Selector(bx, by, VERTICAL_SELECTOR,
	selector_title, initial_selection,
	function,
	button1_title,
	button2_title,
	button3_title,
	button4_title,
	button5_title,
	button6_title,
	button7_title,
	button8_title)
{}
